//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/SampleDesigner/SampleToolBox.cpp
//! @brief     Implements class SampleToolBox
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/SampleDesigner/SampleToolBox.h"
#include "GUI/Views/SampleDesigner/DesignerMimeData.h"
#include "GUI/Views/SampleDesigner/SampleToolBoxTreeWidget.h"
#include <QApplication>
#include <QFocusEvent>
#include <QIcon>
#include <QLineEdit>
#include <QToolBar>
#include <QVBoxLayout>

namespace {

class FilterEdit : public QLineEdit {
public:
    explicit FilterEdit(QWidget* parent = 0) : QLineEdit(parent) { setFocusPolicy(Qt::NoFocus); }

protected:
    virtual void mousePressEvent(QMouseEvent* event)
    {
        if (!hasFocus()) // Explicitly focus on click.
            setFocus(Qt::OtherFocusReason);
        QLineEdit::mousePressEvent(event);
    }

    virtual void focusInEvent(QFocusEvent* e)
    {
        // Refuse the focus if the mouse it outside. In addition to the mouse
        // press logic, this prevents a re-focussing which occurs once
        // we actually had focus
        const Qt::FocusReason reason = e->reason();
        if (reason == Qt::ActiveWindowFocusReason || reason == Qt::PopupFocusReason) {
            const QPoint mousePos = mapFromGlobal(QCursor::pos());
            const bool refuse = !geometry().contains(mousePos);
            if (refuse) {
                e->ignore();
                return;
            }
        }
        QLineEdit::focusInEvent(e);
    }
};

} // namespace


SampleToolBox::SampleToolBox(QWidget* parent) : QWidget(parent), m_view(new SampleToolBoxTreeWidget)
{
    setObjectName(QLatin1String("SampleToolBox"));
    setWindowTitle("Items Toolbox");

    // Prevent the filter from grabbing focus since Our view has Qt::NoFocus
    QToolBar* toolBar = new QToolBar(this);
    QLineEdit* filterEdit = new FilterEdit(toolBar);
    filterEdit->setPlaceholderText("Filter");
    filterEdit->setClearButtonEnabled(true);
    connect(filterEdit, &QLineEdit::textChanged, m_view, &SampleToolBoxTreeWidget::filter);
    toolBar->addWidget(filterEdit);

    QVBoxLayout* layout = new QVBoxLayout(this);
    layout->setMargin(0);
    layout->setSpacing(0);
    layout->addWidget(toolBar);
    layout->addWidget(m_view);
    setLayout(layout);

    connect(m_view, &SampleToolBoxTreeWidget::pressed, this, &SampleToolBox::handleMousePress);

    fill();
}

QSize SampleToolBox::minimumSizeHint() const
{
    return QSize(300, 300);
}

void SampleToolBox::handleMousePress(const QString& /*title*/, const QString& className,
                                     const QPoint&)
{
    if (QApplication::mouseButtons() != Qt::LeftButton)
        return;

    DesignerMimeData::execDrag(className, this);
}

void SampleToolBox::fill()
{
    const auto group = [&](const QString& title) { m_view->addGroup(title); };
    const auto entry = [&](const QString& title, const QString& iconName, const QString& cls,
                           QString tooltip) {
        m_view->addEntry(title, QIcon(":/SampleDesignerToolbox/images/" + iconName), cls, tooltip);
    };

    // ------------------------------------------------------------------------------------------
    group("Sample");
    entry("Multi Layer", "MultiLayer.png", "MultiLayer", "A multilayer to hold stack of layers");

    // ------------------------------------------------------------------------------------------
    group("Layers");
    entry("Layer", "Layer.png", "Layer", "A layer with thickness and material");

    // ------------------------------------------------------------------------------------------
    group("Particle layouts");
    entry("Basic particle layout", "ParticleLayout.png", "ParticleLayout", "A layout of particles");

    // ------------------------------------------------------------------------------------------
    group("Interference functions");
    entry("Radial paracrystal", "ParaCrystal1D.png", "InterferenceRadialParaCrystal",
          "Interference function of radial paracrystal");
    entry("2D paracrystal", "ParaCrystal2D.png", "Interference2DParaCrystal",
          "Interference function of two-dimensional paracrystal");
    entry("1D lattice", "Lattice1D.png", "Interference1DLattice",
          "Interference function of 1D lattice");
    entry("2D lattice", "Lattice2D.png", "Interference2DLattice",
          "Interference function of 2D lattice");
    entry("Finite 2D lattice", "Lattice2DFinite.png", "InterferenceFinite2DLattice",
          "Interference function of finite 2D lattice");
    entry("Hard disk Percus-Yevick", "Lattice2D.png", "InterferenceHardDisk",
          "Interference function for hard disk Percus-Yevick");

    // ------------------------------------------------------------------------------------------
    group("Hard particles");
    entry("Anisotropic pyramid", "ff_AnisoPyramid_64x64.png", "FormFactorAnisoPyramid",
          "Pyramid with a rectangular base");
    entry("Box", "ff_Box_64x64.png", "FormFactorBox", "Rectangular cuboid");
    entry("Cone", "ff_Cone_64x64.png", "FormFactorCone", "Truncated cone");
    entry("Cone6", "ff_Cone6_64x64.png", "FormFactorCone6",
          "A truncated pyramid, based on a regular hexagon");
    entry("Cuboctahedron", "ff_Cuboctahedron_64x64.png", "FormFactorCuboctahedron",
          "Combination of two pyramids of different heights, sharing a common face");
    entry("Cylinder", "ff_Cylinder_64x64.png", "FormFactorCylinder",
          "Cylinder with a circular base");
    entry("Dodecahedron", "ff_Dodecahedron_64x64.png", "FormFactorDodecahedron", "Dodecahedron");
    entry("Ellipsoidal cylinder", "ff_EllipsoidalCylinder_64x64.png",
          "FormFactorEllipsoidalCylinder", "Cylinder with an ellipse cross section");
    entry("Full sphere", "ff_FullSphere_64x64.png", "FormFactorFullSphere", "Full sphere");
    entry("Full spheroid", "ff_FullSpheroid_64x64.png", "FormFactorFullSpheroid",
          "Ellipsoid of revolution");
    entry("Hemi ellipsoid", "ff_HemiEllipsoid_64x64.png", "FormFactorHemiEllipsoid",
          "A horizontally oriented ellipsoid, truncated at the central plane");
    entry("Icosahedron", "ff_Icosahedron_64x64.png", "FormFactorIcosahedron", "Icosahedron");
    entry("Prism3", "ff_Prism3_64x64.png", "FormFactorPrism3",
          "Prism with an equilateral triangle base");
    entry("Prism6", "ff_Prism6_64x64.png", "FormFactorPrism6",
          "Prism with a regular hexagonal base");
    entry("Pyramid", "ff_Pyramid_64x64.png", "FormFactorPyramid",
          "Truncated pyramid with a square base");
    entry("Tetrahedron", "ff_Tetrahedron_64x64.png", "FormFactorTetrahedron",
          "Truncated polyhedron with equilateral triangle base and cropped side faces");
    entry("Truncated cube", "ff_TruncatedCube_64x64.png", "FormFactorTruncatedCube",
          "A cube whose eight vertices have been removed");
    entry("Truncated sphere", "ff_TruncatedSphere_64x64.png", "FormFactorTruncatedSphere",
          "Spherical dome");
    entry("Truncated spheroid", "ff_TruncatedSpheroid_64x64.png", "FormFactorTruncatedSpheroid",
          "Spheroidal dome");

    // ------------------------------------------------------------------------------------------
    group("Ripples");
    entry("Box", "ff_Box_64x64.png", "FormFactorBox", "Rectangular cuboid");
    entry("BarGauss", "ff_Box_64x64.png", "FormFactorBarGauss", "Rectangular cuboid");
    entry("BarLorentz", "ff_Box_64x64.png", "FormFactorBarLorentz", "Rectangular cuboid");
    entry("CosineRippleBox", "ff_Ripple1_64x64.png", "FormFactorCosineRippleBox",
          "Particle with a cosine profile and a rectangular base");
    entry("CosineRippleGauss", "ff_Ripple1_64x64.png", "FormFactorCosineRippleGauss",
          "Particle with a cosine profile and a rectangular base");
    entry("CosineRippleLorentz", "ff_Ripple1_64x64.png", "FormFactorCosineRippleLorentz",
          "Particle with a cosine profile and a rectangular base");
    entry("SawtoothRippleBox", "ff_Ripple2_64x64.png", "FormFactorSawtoothRippleBox",
          "Particle with an asymmetric triangle profile and a rectangular base");
    entry("SawtoothRippleGauss", "ff_Ripple2_64x64.png", "FormFactorSawtoothRippleGauss",
          "Particle with an asymmetric triangle profile and a rectangular base");
    entry("SawtoothRippleLorentz", "ff_Ripple2_64x64.png", "FormFactorSawtoothRippleLorentz",
          "Particle with an asymmetric triangle profile and a rectangular base");

    // ------------------------------------------------------------------------------------------
    group("Transformations");
    entry("Rotation", "Transformation.png", "Rotation", "Rotation applied to particles");

    // ------------------------------------------------------------------------------------------
    group("Particle assemblies");
    entry("Core shell particle", "ParticleCoreShell_64x64.png", "ParticleCoreShell",
          "A particle with a core/shell geometry");
    entry("Particle Composition", "ParticleComposition_64x64.png", "ParticleComposition",
          "Composition of particles with fixed positions");
    entry("Meso Crystal", "Mesocrystal_64x64.png", "MesoCrystal",
          "A 3D crystal structure of nanoparticles");

    // ------------------------------------------------------------------------------------------
    group("Demo samples");
    entry("Cylinder and prisms", "sample_layers2.png", "CylindersAndPrismsBuilder",
          "Mixture of cylinders and prisms without interference");
    entry("Interference 1D paracrystal", "sample_layers2.png", "RadialParaCrystalBuilder",
          "Interference 1D radial paracrystal");
    entry("Interference 2D paracrystal", "sample_layers2.png", "HexParaCrystalBuilder",
          "Interference 2D paracrystal");
    entry("Core shell particles", "sample_layers2.png", "CoreShellParticleBuilder",
          "Core shell particles");
    entry("Multilayer with correlated roughness", "sample_layers2.png",
          "MultiLayerWithRoughnessBuilder", "Multilayer with correlated roughness");
    entry("Interference 2D square lattice", "sample_layers2.png", "SquareLattice2DBuilder",
          "Interference 2D lattice");
    entry("Rotated pyramids", "sample_layers2.png", "RotatedPyramidsBuilder",
          "Rotated pyramids on top of substrate");
    entry("Hexagonal lattice with basis", "sample_layers2.png", "ParticleCompositionBuilder",
          "Hexagonal lattice with basis to represent two layers of spheres");
    entry("Mesocrystal", "sample_layers2.png", "MesoCrystalBuilder", "");
}
